@@ -23,7 +23,6 @@ import datetime
 import functools
 import inspect
 
-from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 import pecan
@@ -36,14 +35,12 @@ from ceilometer.i18n import _, _LI
 from ceilometer import utils
 
 LOG = log.getLogger(__name__)
-cfg.CONF.import_opt('default_api_return_limit', 'ceilometer.api.app',
-                    group='api')
 
 
 def enforce_limit(limit):
     """Ensure limit is defined and is valid. if not, set a default."""
     if limit is None:
-        limit = cfg.CONF.api.default_api_return_limit
+        limit = pecan.request.cfg.api.default_api_return_limit
         LOG.info(_LI('No limit value provided, result set will be'
                      ' limited to %(limit)d.'), {'limit': limit})
     if not limit or limit <= 0:
