@@ -14,7 +14,6 @@
 import os
 
 from jsonpath_rw_ext import parser
-from oslo_config import cfg
 from oslo_log import log
 import six
 import yaml
@@ -26,8 +25,21 @@ LOG = log.getLogger(__name__)
 
 class DefinitionException(Exception):
     def __init__(self, message, definition_cfg):
-        super(DefinitionException, self).__init__(message)
-        self.definition_cfg = definition_cfg
+        msg = '%s %s: %s' % (self.__class__.__name__, definition_cfg, message)
+        super(DefinitionException, self).__init__(msg)
+        self.brief_message = message
+
+
+class MeterDefinitionException(DefinitionException):
+    pass
+
+
+class EventDefinitionException(DefinitionException):
+    pass
+
+
+class ResourceDefinitionException(DefinitionException):
+    pass
 
 
 class Definition(object):
@@ -133,13 +145,13 @@ class Definition(object):
             return getter
 
 
-def load_definitions(defaults, config_file, fallback_file=None):
+def load_definitions(conf, defaults, config_file, fallback_file=None):
     """Setup a definitions from yaml config file."""
 
     if not os.path.exists(config_file):
-        config_file = cfg.CONF.find_file(config_file)
+        config_file = conf.find_file(config_file)
     if not config_file and fallback_file is not None:
-        LOG.debug("No Definitions configuration file found!"
+        LOG.debug("No Definitions configuration file found! "
                   "Using default config.")
         config_file = fallback_file
 
@@ -167,7 +179,7 @@ def load_definitions(defaults, config_file, fallback_file=None):
             raise
 
     else:
-        LOG.debug("No Definitions configuration file found!"
+        LOG.debug("No Definitions configuration file found! "
                   "Using default config.")
         definition_cfg = defaults
 
