@@ -12,10 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import logging
-
-from oslo_config import cfg
-from oslo_context import context
+from oslo_log import log
 import oslo_messaging
 from stevedore import extension
 
@@ -23,15 +20,15 @@ from ceilometer.event import converter as event_converter
 from ceilometer.i18n import _LE
 from ceilometer import messaging
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 class EventsNotificationEndpoint(object):
     def __init__(self, manager):
         super(EventsNotificationEndpoint, self).__init__()
         LOG.debug('Loading event definitions')
-        self.ctxt = context.get_admin_context()
         self.event_converter = event_converter.setup_events(
+            manager.conf,
             extension.ExtensionManager(
                 namespace='ceilometer.event.trait_plugin'))
         self.manager = manager
@@ -61,10 +58,10 @@ class EventsNotificationEndpoint(object):
             try:
                 event = self.event_converter.to_event(notification)
                 if event is not None:
-                    with self.manager.publisher(self.ctxt) as p:
+                    with self.manager.publisher() as p:
                         p(event)
             except Exception:
-                if not cfg.CONF.notification.ack_on_event_error:
+                if not self.manager.conf.notification.ack_on_event_error:
                     return oslo_messaging.NotificationResult.REQUEUE
                 LOG.error(_LE('Fail to process a notification'), exc_info=True)
         return oslo_messaging.NotificationResult.HANDLED
