@@ -15,17 +15,16 @@
 import itertools
 import threading
 
+from concurrent import futures
+from futurist import periodics
 from oslo_config import cfg
-from oslo_context import context
 from oslo_log import log
 import oslo_messaging
 from stevedore import extension
 
-from ceilometer.agent import plugin_base as base
 from ceilometer import coordination
 from ceilometer.event import endpoint as event_endpoint
-from ceilometer import exchange_control
-from ceilometer.i18n import _, _LI, _LW
+from ceilometer.i18n import _, _LI
 from ceilometer import messaging
 from ceilometer import pipeline
 from ceilometer import service_base
@@ -41,21 +40,12 @@ OPTS = [
                min=1,
                help='Number of queues to parallelize workload across. This '
                     'value should be larger than the number of active '
-                    'notification agents for optimal results.'),
+                    'notification agents for optimal results. WARNING: '
+                    'Once set, lowering this value may result in lost data.'),
     cfg.BoolOpt('ack_on_event_error',
                 default=True,
                 deprecated_group='collector',
                 help='Acknowledge message when event persistence fails.'),
-    cfg.BoolOpt('store_events',
-                deprecated_group='collector',
-                default=False,
-                help='Save event details.'),
-    cfg.BoolOpt('disable_non_metric_meters',
-                default=True,
-                help='WARNING: Ceilometer historically offered the ability to '
-                     'store events as meters. This usage is NOT advised as it '
-                     'can flood the metering database and cause performance '
-                     'degradation.'),
     cfg.BoolOpt('workload_partitioning',
                 default=False,
                 help='Enable workload partitioning, allowing multiple '
@@ -64,26 +54,33 @@ OPTS = [
                     default=[],
                     secret=True,
                     help="Messaging URLs to listen for notifications. "
-                         "Example: transport://user:pass@host1:port"
-                         "[,hostN:portN]/virtual_host "
-                         "(DEFAULT/transport_url is used if empty)"),
+                         "Example: rabbit://user:pass@host1:port1"
+                         "[,user:pass@hostN:portN]/virtual_host "
+                         "(DEFAULT/transport_url is used if empty). This "
+                         "is useful when you have dedicate messaging nodes "
+                         "for each service, for example, all nova "
+                         "notifications go to rabbit-nova:5672, while all "
+                         "cinder notifications go to rabbit-cinder:5672."),
     cfg.IntOpt('batch_size',
-               default=1,
+               default=100, min=1,
                help='Number of notification messages to wait before '
-               'publishing them'),
+               'publishing them. Batching is advised when transformations are '
+               'applied in pipeline.'),
     cfg.IntOpt('batch_timeout',
-               default=None,
-               help='Number of seconds to wait before publishing samples'
+               default=5,
+               help='Number of seconds to wait before publishing samples '
                'when batch_size is not reached (None means indefinitely)'),
+    cfg.IntOpt('workers',
+               default=1,
+               min=1,
+               deprecated_group='DEFAULT',
+               deprecated_name='notification_workers',
+               help='Number of workers for notification service, '
+               'default value is 1.')
 ]
 
-cfg.CONF.register_opts(exchange_control.EXCHANGE_OPTS)
-cfg.CONF.register_opts(OPTS, group="notification")
-cfg.CONF.import_opt('telemetry_driver', 'ceilometer.publisher.messaging',
-                    group='publisher_notifier')
 
-
-class NotificationService(service_base.BaseService):
+class NotificationService(service_base.PipelineBasedService):
     """Notification service.
 
     When running multiple agents, additional queuing sequence is required for
@@ -106,18 +103,18 @@ class NotificationService(service_base.BaseService):
 
     def _get_notifiers(self, transport, pipe):
         notifiers = []
-        for x in range(cfg.CONF.notification.pipeline_processing_queues):
+        for x in range(self.conf.notification.pipeline_processing_queues):
             notifiers.append(oslo_messaging.Notifier(
                 transport,
-                driver=cfg.CONF.publisher_notifier.telemetry_driver,
+                driver=self.conf.publisher_notifier.telemetry_driver,
                 publisher_id=pipe.name,
-                topic='%s-%s-%s' % (self.NOTIFICATION_IPC, pipe.name, x)))
+                topics=['%s-%s-%s' % (self.NOTIFICATION_IPC, pipe.name, x)]))
         return notifiers
 
     def _get_pipe_manager(self, transport, pipeline_manager):
 
-        if cfg.CONF.notification.workload_partitioning:
-            pipe_manager = pipeline.SamplePipelineTransportManager()
+        if self.conf.notification.workload_partitioning:
+            pipe_manager = pipeline.SamplePipelineTransportManager(self.conf)
             for pipe in pipeline_manager.pipelines:
                 key = pipeline.get_pipeline_grouping_key(pipe)
                 pipe_manager.add_transporter(
@@ -129,21 +126,22 @@ class NotificationService(service_base.BaseService):
         return pipe_manager
 
     def _get_event_pipeline_manager(self, transport):
+        if self.conf.notification.workload_partitioning:
+            event_pipe_manager = pipeline.EventPipelineTransportManager(
+                self.conf)
+            for pipe in self.event_pipeline_manager.pipelines:
+                event_pipe_manager.add_transporter(
+                    (pipe.source.support_event, ['event_type'],
+                     self._get_notifiers(transport, pipe)))
+        else:
+            event_pipe_manager = self.event_pipeline_manager
 
-        if cfg.CONF.notification.store_events:
-            if cfg.CONF.notification.workload_partitioning:
-                event_pipe_manager = pipeline.EventPipelineTransportManager()
-                for pipe in self.event_pipeline_manager.pipelines:
-                    event_pipe_manager.add_transporter(
-                        (pipe.source.support_event, ['event_type'],
-                         self._get_notifiers(transport, pipe)))
-            else:
-                event_pipe_manager = self.event_pipeline_manager
+        return event_pipe_manager
 
-            return event_pipe_manager
-
-    def start(self):
-        super(NotificationService, self).start()
+    def run(self):
+        super(NotificationService, self).run()
+        self.shutdown = False
+        self.periodic = None
         self.partition_coordinator = None
         self.coord_lock = threading.Lock()
 
@@ -153,17 +151,16 @@ class NotificationService(service_base.BaseService):
         # hence only one listener is required
         self.pipeline_listener = None
 
-        self.pipeline_manager = pipeline.setup_pipeline()
+        self.pipeline_manager = pipeline.setup_pipeline(self.conf)
 
-        if cfg.CONF.notification.store_events:
-            self.event_pipeline_manager = pipeline.setup_event_pipeline()
+        self.event_pipeline_manager = pipeline.setup_event_pipeline(self.conf)
 
-        self.transport = messaging.get_transport()
+        self.transport = messaging.get_transport(self.conf)
 
-        if cfg.CONF.notification.workload_partitioning:
-            self.ctxt = context.get_admin_context()
+        if self.conf.notification.workload_partitioning:
             self.group_id = self.NOTIFICATION_NAMESPACE
-            self.partition_coordinator = coordination.PartitionCoordinator()
+            self.partition_coordinator = coordination.PartitionCoordinator(
+                self.conf)
             self.partition_coordinator.start()
         else:
             # FIXME(sileht): endpoint uses the notification_topics option
@@ -182,24 +179,33 @@ class NotificationService(service_base.BaseService):
         self._configure_main_queue_listeners(self.pipe_manager,
                                              self.event_pipe_manager)
 
-        if cfg.CONF.notification.workload_partitioning:
+        if self.conf.notification.workload_partitioning:
             # join group after all manager set up is configured
             self.partition_coordinator.join_group(self.group_id)
             self.partition_coordinator.watch_group(self.group_id,
                                                    self._refresh_agent)
-            self.tg.add_timer(cfg.CONF.coordination.heartbeat,
-                              self.partition_coordinator.heartbeat)
-            self.tg.add_timer(cfg.CONF.coordination.check_watchers,
-                              self.partition_coordinator.run_watchers)
-            # configure pipelines after all coordination is configured.
-            self._configure_pipeline_listener()
 
-        if not cfg.CONF.notification.disable_non_metric_meters:
-            LOG.warning(_LW('Non-metric meters may be collected. It is highly '
-                            'advisable to disable these meters using '
-                            'ceilometer.conf or the pipeline.yaml'))
-        # Add a dummy thread to have wait() working
-        self.tg.add_timer(604800, lambda: None)
+            @periodics.periodic(spacing=self.conf.coordination.heartbeat,
+                                run_immediately=True)
+            def heartbeat():
+                self.partition_coordinator.heartbeat()
+
+            @periodics.periodic(spacing=self.conf.coordination.check_watchers,
+                                run_immediately=True)
+            def run_watchers():
+                self.partition_coordinator.run_watchers()
+
+            self.periodic = periodics.PeriodicWorker.create(
+                [], executor_factory=lambda:
+                futures.ThreadPoolExecutor(max_workers=10))
+            self.periodic.add(heartbeat)
+            self.periodic.add(run_watchers)
+
+            utils.spawn_thread(self.periodic.start)
+
+            # configure pipelines after all coordination is configured.
+            with self.coord_lock:
+                self._configure_pipeline_listener()
 
         self.init_pipeline_refresh()
 
@@ -210,19 +216,15 @@ class NotificationService(service_base.BaseService):
             LOG.warning(_('Failed to load any notification handlers for %s'),
                         self.NOTIFICATION_NAMESPACE)
 
-        ack_on_error = cfg.CONF.notification.ack_on_event_error
+        ack_on_error = self.conf.notification.ack_on_event_error
 
         endpoints = []
-        if cfg.CONF.notification.store_events:
-            endpoints.append(
-                event_endpoint.EventsNotificationEndpoint(event_pipe_manager))
+        endpoints.append(
+            event_endpoint.EventsNotificationEndpoint(event_pipe_manager))
 
         targets = []
         for ext in notification_manager:
             handler = ext.obj
-            if (cfg.CONF.notification.disable_non_metric_meters and
-                    isinstance(handler, base.NonMetricNotificationBase)):
-                continue
             LOG.debug('Event types from %(name)s: %(type)s'
                       ' (ack_on_error=%(error)s)',
                       {'name': ext.name,
@@ -231,75 +233,80 @@ class NotificationService(service_base.BaseService):
             # NOTE(gordc): this could be a set check but oslo_messaging issue
             # https://bugs.launchpad.net/oslo.messaging/+bug/1398511
             # This ensures we don't create multiple duplicate consumers.
-            for new_tar in handler.get_targets(cfg.CONF):
+            for new_tar in handler.get_targets(self.conf):
                 if new_tar not in targets:
                     targets.append(new_tar)
             endpoints.append(handler)
 
-        urls = cfg.CONF.notification.messaging_urls or [None]
+        urls = self.conf.notification.messaging_urls or [None]
         for url in urls:
-            transport = messaging.get_transport(url)
+            transport = messaging.get_transport(self.conf, url)
+            # NOTE(gordc): ignore batching as we want pull
+            # to maintain sequencing as much as possible.
             listener = messaging.get_batch_notification_listener(
-                transport, targets, endpoints,
-                batch_size=cfg.CONF.notification.batch_size,
-                batch_timeout=cfg.CONF.notification.batch_timeout)
+                transport, targets, endpoints)
             listener.start()
             self.listeners.append(listener)
 
     def _refresh_agent(self, event):
-        self._configure_pipeline_listener()
+        with self.coord_lock:
+            if self.shutdown:
+                # NOTE(sileht): We are going to shutdown we everything will be
+                # stopped, we should not restart them
+                return
+            self._configure_pipeline_listener()
 
     def _configure_pipeline_listener(self):
-        with self.coord_lock:
-            ev_pipes = []
-            if cfg.CONF.notification.store_events:
-                ev_pipes = self.event_pipeline_manager.pipelines
-            pipelines = self.pipeline_manager.pipelines + ev_pipes
-            transport = messaging.get_transport()
-            partitioned = self.partition_coordinator.extract_my_subset(
-                self.group_id,
-                range(cfg.CONF.notification.pipeline_processing_queues))
+        ev_pipes = self.event_pipeline_manager.pipelines
+        pipelines = self.pipeline_manager.pipelines + ev_pipes
+        transport = messaging.get_transport(self.conf)
+        partitioned = self.partition_coordinator.extract_my_subset(
+            self.group_id,
+            range(self.conf.notification.pipeline_processing_queues))
 
-            endpoints = []
-            targets = []
+        endpoints = []
+        targets = []
 
-            for pipe in pipelines:
-                if isinstance(pipe, pipeline.EventPipeline):
-                    endpoints.append(pipeline.EventPipelineEndpoint(self.ctxt,
-                                                                    pipe))
-                else:
-                    endpoints.append(pipeline.SamplePipelineEndpoint(self.ctxt,
-                                                                     pipe))
+        for pipe in pipelines:
+            if isinstance(pipe, pipeline.EventPipeline):
+                endpoints.append(pipeline.EventPipelineEndpoint(pipe))
+            else:
+                endpoints.append(pipeline.SamplePipelineEndpoint(pipe))
 
-            for pipe_set, pipe in itertools.product(partitioned, pipelines):
-                LOG.debug('Pipeline endpoint: %s from set: %s',
-                          pipe.name, pipe_set)
-                topic = '%s-%s-%s' % (self.NOTIFICATION_IPC,
-                                      pipe.name, pipe_set)
-                targets.append(oslo_messaging.Target(topic=topic))
+        for pipe_set, pipe in itertools.product(partitioned, pipelines):
+            LOG.debug('Pipeline endpoint: %s from set: %s',
+                      pipe.name, pipe_set)
+            topic = '%s-%s-%s' % (self.NOTIFICATION_IPC,
+                                  pipe.name, pipe_set)
+            targets.append(oslo_messaging.Target(topic=topic))
 
-            if self.pipeline_listener:
-                self.pipeline_listener.stop()
-                self.pipeline_listener.wait()
+        if self.pipeline_listener:
+            self.pipeline_listener.stop()
+            self.pipeline_listener.wait()
 
-            self.pipeline_listener = messaging.get_batch_notification_listener(
-                transport,
-                targets,
-                endpoints,
-                batch_size=cfg.CONF.notification.batch_size,
-                batch_timeout=cfg.CONF.notification.batch_timeout)
-            self.pipeline_listener.start()
+        self.pipeline_listener = messaging.get_batch_notification_listener(
+            transport,
+            targets,
+            endpoints,
+            batch_size=self.conf.notification.batch_size,
+            batch_timeout=self.conf.notification.batch_timeout)
+        # NOTE(gordc): set single thread to process data sequentially
+        # if batching enabled.
+        batch = (1 if self.conf.notification.batch_size > 1 else None)
+        self.pipeline_listener.start(override_pool_size=batch)
 
-    def stop(self):
-        if getattr(self, 'partition_coordinator', None):
+    def terminate(self):
+        self.shutdown = True
+        if self.periodic:
+            self.periodic.stop()
+            self.periodic.wait()
+        if self.partition_coordinator:
             self.partition_coordinator.stop()
-        listeners = []
-        if getattr(self, 'listeners', None):
-            listeners.extend(self.listeners)
-        if getattr(self, 'pipeline_listener', None):
-            listeners.append(self.pipeline_listener)
-        utils.kill_listeners(listeners)
-        super(NotificationService, self).stop()
+        with self.coord_lock:
+            if self.pipeline_listener:
+                utils.kill_listeners([self.pipeline_listener])
+            utils.kill_listeners(self.listeners)
+        super(NotificationService, self).terminate()
 
     def reload_pipeline(self):
         LOG.info(_LI("Reloading notification agent and listeners."))
@@ -312,12 +319,18 @@ class NotificationService(service_base.BaseService):
             self.event_pipe_manager = self._get_event_pipeline_manager(
                 self.transport)
 
-        # re-start the main queue listeners.
-        utils.kill_listeners(self.listeners)
-        self._configure_main_queue_listeners(
-            self.pipe_manager, self.event_pipe_manager)
+        with self.coord_lock:
+            if self.shutdown:
+                # NOTE(sileht): We are going to shutdown we everything will be
+                # stopped, we should not restart them
+                return
 
-        # re-start the pipeline listeners if workload partitioning
-        # is enabled.
-        if cfg.CONF.notification.workload_partitioning:
-            self._configure_pipeline_listener()
+            # restart the main queue listeners.
+            utils.kill_listeners(self.listeners)
+            self._configure_main_queue_listeners(
+                self.pipe_manager, self.event_pipe_manager)
+
+            # restart the pipeline listeners if workload partitioning
+            # is enabled.
+            if self.conf.notification.workload_partitioning:
+                self._configure_pipeline_listener()
