@@ -16,12 +16,13 @@
 
 import abc
 
+from debtcollector import removals
 from oslo_utils import netutils
 import six
 from stevedore import driver
 
 
-def get_publisher(url, namespace='ceilometer.publisher'):
+def get_publisher(conf, url, namespace):
     """Get publisher driver and load it.
 
     :param URL: URL for the publisher
@@ -29,20 +30,47 @@ def get_publisher(url, namespace='ceilometer.publisher'):
     """
     parse_result = netutils.urlsplit(url)
     loaded_driver = driver.DriverManager(namespace, parse_result.scheme)
-    return loaded_driver.driver(parse_result)
+    if issubclass(loaded_driver.driver, ConfigPublisherBase):
+        return loaded_driver.driver(conf, parse_result)
+    else:
+        return loaded_driver.driver(parse_result)
 
 
+@removals.removed_class("PublisherBase",
+                        message="Use ConfigPublisherBase instead",
+                        removal_version="9.0.0")
 @six.add_metaclass(abc.ABCMeta)
 class PublisherBase(object):
-    """Base class for plugins that publish data."""
+    """Legacy base class for plugins that publish data.
+
+    This base class is for backward compatibility purpose. It doesn't take
+    oslo.config object as argument. We assume old publisher does not depend
+    on cfg.CONF.
+    """
 
     def __init__(self, parsed_url):
         pass
 
     @abc.abstractmethod
-    def publish_samples(self, context, samples):
+    def publish_samples(self, samples):
         """Publish samples into final conduit."""
 
     @abc.abstractmethod
-    def publish_events(self, context, events):
+    def publish_events(self, events):
+        """Publish events into final conduit."""
+
+
+@six.add_metaclass(abc.ABCMeta)
+class ConfigPublisherBase(object):
+    """Base class for plugins that publish data."""
+
+    def __init__(self, conf, parsed_url):
+        self.conf = conf
+
+    @abc.abstractmethod
+    def publish_samples(self, samples):
+        """Publish samples into final conduit."""
+
+    @abc.abstractmethod
+    def publish_events(self, events):
         """Publish events into final conduit."""
