@@ -39,8 +39,9 @@ class BinTestCase(base.BaseTestCase):
         super(BinTestCase, self).tearDown()
         os.remove(self.tempfile)
 
-    def test_dbsync_run(self):
-        subp = subprocess.Popen(['ceilometer-dbsync',
+    def test_upgrade_run(self):
+        subp = subprocess.Popen(['ceilometer-upgrade',
+                                 '--skip-gnocchi-resource-types',
                                  "--config-file=%s" % self.tempfile])
         self.assertEqual(0, subp.wait())
 
@@ -53,8 +54,6 @@ class BinTestCase(base.BaseTestCase):
         self.assertEqual(0, subp.poll())
         self.assertIn(b"Nothing to clean, database metering "
                       b"time to live is disabled", stdout)
-        self.assertIn(b"Nothing to clean, database event "
-                      b"time to live is disabled", stdout)
 
     def _test_run_expirer_ttl_enabled(self, ttl_name, data_name):
         content = ("[DEFAULT]\n"
@@ -82,7 +81,6 @@ class BinTestCase(base.BaseTestCase):
         self._test_run_expirer_ttl_enabled('metering_time_to_live',
                                            'metering')
         self._test_run_expirer_ttl_enabled('time_to_live', 'metering')
-        self._test_run_expirer_ttl_enabled('event_time_to_live', 'event')
 
 
 class BinSendSampleTestCase(base.BaseTestCase):
