@@ -25,10 +25,11 @@ import six
 from stevedore import extension
 import yaml
 
-from ceilometer.compute.notifications import instance
+from ceilometer.agent import plugin_base
 from ceilometer import messaging
 from ceilometer import notification
 from ceilometer.publisher import test as test_publisher
+from ceilometer import service
 from ceilometer.tests import base as tests_base
 
 TEST_NOTICE_CTXT = {
@@ -83,22 +84,33 @@ TEST_NOTICE_PAYLOAD = {
 }
 
 
+class _FakeNotificationPlugin(plugin_base.NotificationBase):
+    event_types = ['fake.event']
+
+    def get_targets(self, conf):
+        return [oslo_messaging.Target(
+            topic=topic, exchange=conf.nova_control_exchange)
+            for topic in self.get_notification_topics(conf)]
+
+    def process_notification(self, message):
+        return []
+
+
 class TestNotification(tests_base.BaseTestCase):
 
     def setUp(self):
         super(TestNotification, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
+        conf = service.prepare_service([], [])
+        self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.CONF.set_override("connection", "log://", group='database')
         self.CONF.set_override("backend_url", None, group="coordination")
-        self.CONF.set_override("disable_non_metric_meters", False,
-                               group="notification")
         self.CONF.set_override("workload_partitioning", True,
                                group='notification')
         self.setup_messaging(self.CONF)
-        self.srv = notification.NotificationService(0)
+        self.srv = notification.NotificationService(0, self.CONF)
 
     def fake_get_notifications_manager(self, pm):
-        self.plugin = instance.Instance(pm)
+        self.plugin = _FakeNotificationPlugin(pm)
         return extension.ExtensionManager.make_test_instance(
             [
                 extension.Extension('test',
@@ -152,7 +164,7 @@ class TestNotification(tests_base.BaseTestCase):
     def test_unique_consumers(self, mock_listener):
 
         def fake_get_notifications_manager_dup_targets(pm):
-            plugin = instance.Instance(pm)
+            plugin = _FakeNotificationPlugin(pm)
             return extension.ExtensionManager.make_test_instance(
                 [extension.Extension('test', None, None, plugin),
                  extension.Extension('test', None, None, plugin)])
@@ -212,19 +224,16 @@ class BaseRealNotification(tests_base.BaseTestCase):
 
     def setUp(self):
         super(BaseRealNotification, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
-        # Dummy config file to avoid looking for system config
-        self.CONF([], project='ceilometer', validate_default_values=True)
+        conf = service.prepare_service([], [])
+        self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.setup_messaging(self.CONF, 'nova')
 
-        pipeline_cfg_file = self.setup_pipeline(['instance', 'memory'])
+        pipeline_cfg_file = self.setup_pipeline(['vcpus', 'memory'])
         self.CONF.set_override("pipeline_cfg_file", pipeline_cfg_file)
 
         self.expected_samples = 2
 
         self.CONF.set_override("backend_url", None, group="coordination")
-        self.CONF.set_override("disable_non_metric_meters", False,
-                               group="notification")
 
         ev_pipeline_cfg_file = self.setup_event_pipeline(
             ['compute.instance.*'])
@@ -236,7 +245,7 @@ class BaseRealNotification(tests_base.BaseTestCase):
             "definitions_cfg_file",
             self.path_get('etc/ceilometer/event_definitions.yaml'),
             group='event')
-        self.publisher = test_publisher.TestPublisher("")
+        self.publisher = test_publisher.TestPublisher(self.CONF, "")
 
     def _check_notification_service(self):
         self.srv.run()
@@ -265,7 +274,7 @@ class TestRealNotificationReloadablePipeline(BaseRealNotification):
         self.CONF.set_override('refresh_pipeline_cfg', True)
         self.CONF.set_override('refresh_event_pipeline_cfg', True)
         self.CONF.set_override('pipeline_polling_interval', 1)
-        self.srv = notification.NotificationService(0)
+        self.srv = notification.NotificationService(0, self.CONF)
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_pipeline_poller(self, fake_publisher_cls):
@@ -325,7 +334,7 @@ class TestRealNotification(BaseRealNotification):
 
     def setUp(self):
         super(TestRealNotification, self).setUp()
-        self.srv = notification.NotificationService(0)
+        self.srv = notification.NotificationService(0, self.CONF)
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_service(self, fake_publisher_cls):
@@ -347,16 +356,6 @@ class TestRealNotification(BaseRealNotification):
                 break
         self.assertEqual(self.expected_events, len(self.publisher.events))
 
-    @mock.patch('ceilometer.publisher.test.TestPublisher')
-    def test_notification_disable_non_metrics(self, fake_publisher_cls):
-        self.CONF.set_override("disable_non_metric_meters", True,
-                               group="notification")
-        # instance is a not a metric. we should only get back memory
-        self.expected_samples = 1
-        fake_publisher_cls.return_value = self.publisher
-        self._check_notification_service()
-        self.assertEqual('memory', self.publisher.samples[0].name)
-
 
 class TestRealNotificationHA(BaseRealNotification):
 
@@ -364,7 +363,7 @@ class TestRealNotificationHA(BaseRealNotification):
         super(TestRealNotificationHA, self).setUp()
         self.CONF.set_override('workload_partitioning', True,
                                group='notification')
-        self.srv = notification.NotificationService(0)
+        self.srv = notification.NotificationService(0, self.CONF)
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_service(self, fake_publisher_cls):
@@ -492,7 +491,7 @@ class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
             'sources': [{
                 'name': 'test_pipeline',
                 'interval': 5,
-                'meters': ['instance', 'memory'],
+                'meters': ['vcpus', 'memory'],
                 'sinks': ['test_sink']
             }],
             'sinks': [{
@@ -523,8 +522,8 @@ class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
 
     def setUp(self):
         super(TestRealNotificationMultipleAgents, self).setUp()
-        self.CONF = self.useFixture(fixture_config.Config()).conf
-        self.CONF([], project='ceilometer', validate_default_values=True)
+        conf = service.prepare_service([], [])
+        self.CONF = self.useFixture(fixture_config.Config(conf)).conf
         self.setup_messaging(self.CONF, 'nova')
 
         pipeline_cfg_file = self.setup_pipeline(['instance', 'memory'])
@@ -533,20 +532,18 @@ class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
         self.CONF.set_override("event_pipeline_cfg_file",
                                event_pipeline_cfg_file)
         self.CONF.set_override("backend_url", None, group="coordination")
-        self.CONF.set_override("disable_non_metric_meters", False,
-                               group="notification")
         self.CONF.set_override('workload_partitioning', True,
                                group='notification')
         self.CONF.set_override('pipeline_processing_queues', 2,
                                group='notification')
-        self.publisher = test_publisher.TestPublisher("")
-        self.publisher2 = test_publisher.TestPublisher("")
+        self.publisher = test_publisher.TestPublisher(self.CONF, "")
+        self.publisher2 = test_publisher.TestPublisher(self.CONF, "")
 
     def _check_notifications(self, fake_publisher_cls):
         fake_publisher_cls.side_effect = [self.publisher, self.publisher2]
 
-        self.srv = notification.NotificationService(0)
-        self.srv2 = notification.NotificationService(0)
+        self.srv = notification.NotificationService(0, self.CONF)
+        self.srv2 = notification.NotificationService(0, self.CONF)
         with mock.patch('ceilometer.coordination.PartitionCoordinator'
                         '._get_members', return_value=['harry', 'lloyd']):
             with mock.patch('uuid.uuid4', return_value='harry'):
