@@ -29,7 +29,7 @@ class TestTestNotifierHook(base.BaseTestCase):
     def test_init_notifier_with_drivers(self):
         self.CONF.set_override('telemetry_driver', 'messagingv2',
                                group='publisher_notifier')
-        hook = hooks.NotifierHook()
+        hook = hooks.NotifierHook(self.CONF)
         notifier = hook.notifier
         self.assertIsInstance(notifier, oslo_messaging.Notifier)
         self.assertEqual(['messagingv2'], notifier._driver_names)
