@@ -24,7 +24,6 @@ from oslotest import mockpatch
 import wsme
 
 from ceilometer.api.controllers.v2 import base as v2_base
-from ceilometer.api.controllers.v2 import events
 from ceilometer.api.controllers.v2 import meters
 from ceilometer.api.controllers.v2 import utils
 from ceilometer import storage
@@ -37,10 +36,6 @@ class TestQuery(base.BaseTestCase):
         super(TestQuery, self).setUp()
         self.useFixture(fixtures.MonkeyPatch(
             'pecan.response', mock.MagicMock()))
-        self.useFixture(mockpatch.Patch('ceilometer.api.controllers.v2.events'
-                                        '._build_rbac_query_filters',
-                                        return_value={'t_filter': [],
-                                                      'admin_proj': None}))
 
     def test_get_value_as_type_with_integer(self):
         query = v2_base.Query(field='metadata.size',
@@ -158,15 +153,6 @@ class TestQuery(base.BaseTestCase):
         expected = value
         self.assertEqual(expected, query._get_value_as_type())
 
-    def test_event_query_to_event_filter_with_bad_op(self):
-        # bug 1511592
-        query = v2_base.Query(field='event_type',
-                              op='ne',
-                              value='compute.instance.create.end',
-                              type='string')
-        self.assertRaises(v2_base.ClientSideError,
-                          events._event_query_to_event_filter, [query])
-
 
 class TestValidateGroupByFields(base.BaseTestCase):
 
