@@ -17,7 +17,6 @@ import os
 import uuid
 
 from gnocchiclient import exceptions as gnocchi_exc
-from gnocchiclient import utils as gnocchi_utils
 from keystoneauth1 import exceptions as ka_exceptions
 import mock
 from oslo_config import fixture as config_fixture
@@ -185,13 +184,136 @@ IMAGE_DELETE_START = {
 }
 
 
+VOLUME_DELETE_START = {
+    'event_type': u'volume.delete.start',
+    'traits': [
+        [
+            u'availability_zone',
+            1,
+            u'nova'
+        ],
+        [
+            u'created_at',
+            1,
+            u'2016-11-28T13:19:53+00:00'
+        ],
+        [
+            u'display_name',
+            1,
+            u'vol-001'
+        ],
+        [
+            u'host',
+            1,
+            u'zhangguoqing-dev@lvmdriver-1#lvmdriver-1'
+        ],
+        [
+            u'project_id',
+            1,
+            u'd53fcc7dc53c4662ad77822c36a21f00'
+        ],
+        [
+            u'replication_status',
+            1,
+            u'disabled'
+        ],
+        [
+            u'request_id',
+            1,
+            u'req-f44df096-50d4-4211-95ea-64be6f5e4f60'
+        ],
+        [
+            u'resource_id',
+            1,
+            u'6cc6e7dd-d17d-460f-ae79-7e08a216ce96'
+        ],
+        [
+            u'service',
+            1,
+            u'volume.zhangguoqing-dev@lvmdriver-1'
+        ],
+        [
+            u'size',
+            1,
+            u'1'
+        ],
+        [
+            u'status',
+            1,
+            u'deleting'
+        ],
+        [
+            u'tenant_id',
+            1,
+            u'd53fcc7dc53c4662ad77822c36a21f00'
+        ],
+        [
+            u'type',
+            1,
+            u'af6271fa-13c4-44e6-9246-754ce9dc7df8'
+        ],
+        [
+            u'user_id',
+            1,
+            u'819bbd28f5374506b8502521c89430b5'
+        ]
+    ],
+    'message_signature':
+        '831719d54059734f82e7d6498c6d7a8fd637568732e79c1fd375e128f142373a',
+    'raw': {},
+    'generated': '2016-11-28T13:42:15.484674',
+    'message_id': u'a15b94ee-cb8e-4c71-9abe-14aa80055fb4'
+}
+
+FLOATINGIP_DELETE_END = {
+    'event_type': u'floatingip.delete.end',
+    'traits': [
+        [
+            u'project_id',
+            1,
+            u'd53fcc7dc53c4662ad77822c36a21f00'
+        ],
+        [
+            u'request_id',
+            1,
+            'req-443ddb77-31f7-41fe-abbf-921107dd9f00'
+        ],
+        [
+            u'resource_id',
+            1,
+            u'705e2c08-08e8-45cb-8673-5c5be955569b'
+        ],
+        [
+            u'service',
+            1,
+            u'network.zhangguoqing-dev'
+        ],
+        [
+            u'tenant_id',
+            1,
+            u'd53fcc7dc53c4662ad77822c36a21f00'
+        ],
+        [
+            u'user_id',
+            1,
+            u'819bbd28f5374506b8502521c89430b5'
+        ]
+    ],
+    'message_signature':
+        '831719d54059734f82e7d6498c6d7a8fd637568732e79c1fd375e128f142373a',
+    'raw': {},
+    'generated': '2016-11-29T09:25:55.474710',
+    'message_id': u'a15b94ee-cb8e-4c71-9abe-14aa80055fb4'
+}
+
+
 @mock.patch('gnocchiclient.v1.client.Client', mock.Mock())
 class DispatcherTest(base.BaseTestCase):
 
     def setUp(self):
         super(DispatcherTest, self).setUp()
-        self.conf = self.useFixture(config_fixture.Config())
-        ceilometer_service.prepare_service(argv=[], config_files=[])
+        conf = ceilometer_service.prepare_service(argv=[], config_files=[])
+        self.conf = self.useFixture(config_fixture.Config(conf))
         self.conf.config(
             resources_definition_file=self.path_get(
                 'etc/ceilometer/gnocchi_resources.yaml'),
@@ -406,14 +528,18 @@ class DispatcherWorkflowTest(base.BaseTestCase,
                 'display_name': 'myinstance',
             },
             metric_names=[
-                'instance', 'disk.root.size', 'disk.ephemeral.size',
+                'disk.root.size', 'disk.ephemeral.size',
                 'memory', 'vcpus', 'memory.usage', 'memory.resident',
+                'memory.bandwidth.total', 'memory.bandwidth.local',
                 'cpu', 'cpu.delta', 'cpu_util', 'vcpus', 'disk.read.requests',
+                'cpu_l3_cache', 'perf.cpu.cycles', 'perf.instructions',
+                'perf.cache.references', 'perf.cache.misses',
                 'disk.read.requests.rate', 'disk.write.requests',
                 'disk.write.requests.rate', 'disk.read.bytes',
                 'disk.read.bytes.rate', 'disk.write.bytes',
                 'disk.write.bytes.rate', 'disk.latency', 'disk.iops',
-                'disk.capacity', 'disk.allocation', 'disk.usage'],
+                'disk.capacity', 'disk.allocation', 'disk.usage',
+                'compute.instance.booting.time'],
             resource_type='instance')),
         ('hardware.ipmi.node.power', dict(
             sample={
@@ -452,21 +578,20 @@ class DispatcherWorkflowTest(base.BaseTestCase,
     ]
 
     default_workflow = dict(resource_exists=True,
-                            metric_exists=True,
                             post_measure_fail=False,
                             create_resource_fail=False,
-                            create_metric_fail=False,
+                            create_resource_race=False,
                             update_resource_fail=False,
                             retry_post_measures_fail=False)
     workflow_scenarios = [
         ('normal_workflow', {}),
         ('new_resource', dict(resource_exists=False)),
+        ('new_resource_compat', dict(resource_exists=False)),
         ('new_resource_fail', dict(resource_exists=False,
                                    create_resource_fail=True)),
+        ('new_resource_race', dict(resource_exists=False,
+                                   create_resource_race=True)),
         ('resource_update_fail', dict(update_resource_fail=True)),
-        ('new_metric', dict(metric_exists=False)),
-        ('new_metric_fail', dict(metric_exists=False,
-                                 create_metric_fail=True)),
         ('retry_fail', dict(resource_exists=False,
                             retry_post_measures_fail=True)),
         ('measure_fail', dict(post_measure_fail=True)),
@@ -484,11 +609,8 @@ class DispatcherWorkflowTest(base.BaseTestCase,
 
     def setUp(self):
         super(DispatcherWorkflowTest, self).setUp()
-        self.conf = self.useFixture(config_fixture.Config())
-        # Set this explicitly to avoid conflicts with any existing
-        # configuration.
-        self.conf.config(url='http://localhost:8041',
-                         group='dispatcher_gnocchi')
+        conf = ceilometer_service.prepare_service(argv=[], config_files=[])
+        self.conf = self.useFixture(config_fixture.Config(conf))
         ks_client = mock.Mock()
         ks_client.projects.find.return_value = mock.Mock(
             name='gnocchi', id='a2d42c23-d518-46b6-96ab-3fba2e146859')
@@ -497,14 +619,13 @@ class DispatcherWorkflowTest(base.BaseTestCase,
             return_value=ks_client))
         self.ks_client = ks_client
 
-        ceilometer_service.prepare_service(argv=[], config_files=[])
         self.conf.config(
             resources_definition_file=self.path_get(
                 'etc/ceilometer/gnocchi_resources.yaml'),
             group="dispatcher_gnocchi"
         )
 
-        self.sample['resource_id'] = str(uuid.uuid4()) + "/foobar"
+        self.sample['resource_id'] = str(uuid.uuid4()) + "_foobar"
         self.sample['message_signature'] = utils.compute_signature(
             self.sample, self.conf.conf.publisher.telemetry_secret)
 
@@ -548,12 +669,20 @@ class DispatcherWorkflowTest(base.BaseTestCase,
                 {'ended_at': now.isoformat()}),
             mock.call.resource.update(
                 'image', 'dc337359-de70-4044-8e2c-80573ba6e577',
+                {'ended_at': now.isoformat()}),
+            mock.call.resource.update(
+                'volume', '6cc6e7dd-d17d-460f-ae79-7e08a216ce96',
+                {'ended_at': now.isoformat()}),
+            mock.call.resource.update(
+                'network', '705e2c08-08e8-45cb-8673-5c5be955569b',
                 {'ended_at': now.isoformat()})
         ]
 
         self.dispatcher.record_events([INSTANCE_DELETE_START,
-                                      IMAGE_DELETE_START])
-        self.assertEqual(7, len(fakeclient.mock_calls))
+                                      IMAGE_DELETE_START,
+                                      VOLUME_DELETE_START,
+                                      FLOATINGIP_DELETE_END])
+        self.assertEqual(9, len(fakeclient.mock_calls))
         for call in expected_calls:
             self.assertIn(call, fakeclient.mock_calls)
 
@@ -564,19 +693,15 @@ class DispatcherWorkflowTest(base.BaseTestCase,
 
         fakeclient = fakeclient_cls.return_value
 
-        # FIXME(sileht): we don't use urlparse.quote here
-        # to ensure / is converted in %2F
-        # temporary disabled until we find a solution
-        # on gnocchi side. Current gnocchiclient doesn't
-        # encode the resource_id
-        resource_id = self.sample['resource_id']  # .replace("/", "%2F"),
+        resource_id = self.sample['resource_id'].replace("/", "_")
         metric_name = self.sample['counter_name']
-        gnocchi_id = gnocchi_utils.encode_resource_id(resource_id)
+        gnocchi_id = uuid.uuid4()
 
         expected_calls = [
             mock.call.capabilities.list(),
             mock.call.metric.batch_resources_metrics_measures(
-                {gnocchi_id: {metric_name: self.measures_attributes}})
+                {resource_id: {metric_name: self.measures_attributes}},
+                create_metrics=True)
         ]
         expected_debug = [
             mock.call('gnocchi project found: %s',
@@ -587,11 +712,14 @@ class DispatcherWorkflowTest(base.BaseTestCase,
         batch_side_effect = []
         if self.post_measure_fail:
             batch_side_effect += [Exception('boom!')]
-        elif not self.resource_exists or not self.metric_exists:
+        elif not self.resource_exists:
             batch_side_effect += [
                 gnocchi_exc.BadRequest(
-                    400, "Unknown metrics: %s/%s" % (gnocchi_id,
-                                                     metric_name))]
+                    400, {"cause": "Unknown resources",
+                          'detail': [{
+                              'resource_id': gnocchi_id,
+                              'original_resource_id': resource_id}]})]
+
             attributes = self.postable_attributes.copy()
             attributes.update(self.patchable_attributes)
             attributes['id'] = self.sample['resource_id']
@@ -609,30 +737,18 @@ class DispatcherWorkflowTest(base.BaseTestCase,
 
             if self.create_resource_fail:
                 fakeclient.resource.create.side_effect = [Exception('boom!')]
-            elif self.resource_exists:
+            elif self.create_resource_race:
                 fakeclient.resource.create.side_effect = [
                     gnocchi_exc.ResourceAlreadyExists(409)]
-
-                expected_calls.append(mock.call.metric.create({
-                    'name': self.sample['counter_name'],
-                    'unit': self.sample['counter_unit'],
-                    'resource_id': resource_id}))
-                if self.create_metric_fail:
-                    fakeclient.metric.create.side_effect = [Exception('boom!')]
-                elif self.metric_exists:
-                    fakeclient.metric.create.side_effect = [
-                        gnocchi_exc.NamedMetricAlreadyExists(409)]
-                else:
-                    fakeclient.metric.create.side_effect = [None]
-
             else:  # not resource_exists
                 expected_debug.append(mock.call(
                     'Resource %s created', self.sample['resource_id']))
 
-            if not self.create_resource_fail and not self.create_metric_fail:
+            if not self.create_resource_fail:
                 expected_calls.append(
                     mock.call.metric.batch_resources_metrics_measures(
-                        {gnocchi_id: {metric_name: self.measures_attributes}})
+                        {resource_id: {metric_name: self.measures_attributes}},
+                        create_metrics=True)
                 )
 
                 if self.retry_post_measures_fail:
@@ -668,7 +784,7 @@ class DispatcherWorkflowTest(base.BaseTestCase,
         self.dispatcher.record_metering_data([self.sample])
 
         # Check that the last log message is the expected one
-        if (self.post_measure_fail or self.create_metric_fail
+        if (self.post_measure_fail
                 or self.create_resource_fail
                 or self.retry_post_measures_fail
                 or (self.update_resource_fail and self.patchable_attributes)):
