@@ -15,8 +15,7 @@
 
 import mock
 
-from oslo_config import cfg
-from oslo_context import context
+from oslo_config import fixture as fixture_config
 from oslotest import base
 from oslotest import mockpatch
 
@@ -32,11 +31,11 @@ class _BaseTestLBPollster(base.BaseTestCase):
     def setUp(self):
         super(_BaseTestLBPollster, self).setUp()
         self.addCleanup(mock.patch.stopall)
-        self.context = context.get_admin_context()
-        self.manager = manager.AgentManager()
-        cfg.CONF.set_override('neutron_lbaas_version',
-                              'v1',
-                              group='service_types')
+        self.CONF = self.useFixture(fixture_config.Config()).conf
+        self.manager = manager.AgentManager(0, self.CONF)
+        self.CONF.set_override('neutron_lbaas_version',
+                               'v1',
+                               group='service_types')
         plugin_base._get_keystone = mock.Mock()
         catalog = (plugin_base._get_keystone.session.auth.get_access.
                    return_value.service_catalog)
@@ -48,7 +47,7 @@ class TestLBPoolPollster(_BaseTestLBPollster):
 
     def setUp(self):
         super(TestLBPoolPollster, self).setUp()
-        self.pollster = lbaas.LBPoolPollster()
+        self.pollster = lbaas.LBPoolPollster(self.CONF)
         fake_pools = self.fake_get_pools()
         self.useFixture(mockpatch.Patch('ceilometer.neutron_client.Client.'
                                         'pool_get_all',
@@ -158,7 +157,8 @@ class TestLBPoolPollster(_BaseTestLBPollster):
                          set([s.name for s in samples]))
 
     def test_pool_discovery(self):
-        discovered_pools = discovery.LBPoolsDiscovery().discover(self.manager)
+        discovered_pools = discovery.LBPoolsDiscovery(
+            self.CONF).discover(self.manager)
         self.assertEqual(4, len(discovered_pools))
         for pool in self.fake_get_pools():
             if pool['status'] == 'error':
@@ -171,7 +171,7 @@ class TestLBVipPollster(_BaseTestLBPollster):
 
     def setUp(self):
         super(TestLBVipPollster, self).setUp()
-        self.pollster = lbaas.LBVipPollster()
+        self.pollster = lbaas.LBVipPollster(self.CONF)
         fake_vips = self.fake_get_vips()
         self.useFixture(mockpatch.Patch('ceilometer.neutron_client.Client.'
                                         'vip_get_all',
@@ -281,7 +281,8 @@ class TestLBVipPollster(_BaseTestLBPollster):
                          set([s.name for s in samples]))
 
     def test_vip_discovery(self):
-        discovered_vips = discovery.LBVipsDiscovery().discover(self.manager)
+        discovered_vips = discovery.LBVipsDiscovery(
+            self.CONF).discover(self.manager)
         self.assertEqual(4, len(discovered_vips))
         for pool in self.fake_get_vips():
             if pool['status'] == 'error':
@@ -294,7 +295,7 @@ class TestLBMemberPollster(_BaseTestLBPollster):
 
     def setUp(self):
         super(TestLBMemberPollster, self).setUp()
-        self.pollster = lbaas.LBMemberPollster()
+        self.pollster = lbaas.LBMemberPollster(self.CONF)
         fake_members = self.fake_get_members()
         self.useFixture(mockpatch.Patch('ceilometer.neutron_client.Client.'
                                         'member_get_all',
@@ -374,8 +375,8 @@ class TestLBMemberPollster(_BaseTestLBPollster):
                          set([s.name for s in samples]))
 
     def test_members_discovery(self):
-        discovered_members = discovery.LBMembersDiscovery().discover(
-            self.manager)
+        discovered_members = discovery.LBMembersDiscovery(
+            self.CONF).discover(self.manager)
         self.assertEqual(4, len(discovered_members))
         for pool in self.fake_get_members():
             if pool['status'] == 'error':
@@ -388,7 +389,7 @@ class TestLBHealthProbePollster(_BaseTestLBPollster):
 
     def setUp(self):
         super(TestLBHealthProbePollster, self).setUp()
-        self.pollster = lbaas.LBHealthMonitorPollster()
+        self.pollster = lbaas.LBHealthMonitorPollster(self.CONF)
         fake_health_monitor = self.fake_get_health_monitor()
         self.useFixture(mockpatch.Patch('ceilometer.neutron_client.Client.'
                                         'health_monitor_get_all',
@@ -423,8 +424,8 @@ class TestLBHealthProbePollster(_BaseTestLBPollster):
                          set([s.name for s in samples]))
 
     def test_probes_discovery(self):
-        discovered_probes = discovery.LBHealthMonitorsDiscovery().discover(
-            self.manager)
+        discovered_probes = discovery.LBHealthMonitorsDiscovery(
+            self.CONF).discover(self.manager)
         self.assertEqual(discovered_probes, self.fake_get_health_monitor())
 
 
@@ -473,7 +474,7 @@ class TestLBStatsPollster(_BaseTestLBPollster):
     @mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
     def _check_get_samples(self, factory, sample_name, expected_volume,
                            expected_type):
-        pollster = factory()
+        pollster = factory(self.CONF)
         cache = {}
         samples = list(pollster.get_samples(self.manager, cache,
                                             self.fake_get_pools()))
