@@ -36,21 +36,26 @@ from oslo_utils import timeutils
 from oslo_utils import units
 import six
 
+ROOTWRAP_CONF = "/etc/ceilometer/rootwrap.conf"
 
 OPTS = [
     cfg.StrOpt('rootwrap_config',
-               default="/etc/ceilometer/rootwrap.conf",
-               help='Path to the rootwrap configuration file to'
+               default=ROOTWRAP_CONF,
+               help='Path to the rootwrap configuration file to '
                     'use for running commands as root'),
 ]
-CONF = cfg.CONF
-CONF.register_opts(OPTS)
 
 EPOCH_TIME = datetime.datetime(1970, 1, 1)
 
 
 def _get_root_helper():
-    return 'sudo ceilometer-rootwrap %s' % CONF.rootwrap_config
+    global ROOTWRAP_CONF
+    return 'sudo ceilometer-rootwrap %s' % ROOTWRAP_CONF
+
+
+def setup_root_helper(conf):
+    global ROOTWRAP_CONF
+    ROOTWRAP_CONF = conf.rootwrap_config
 
 
 def execute(*cmd, **kwargs):
@@ -144,13 +149,6 @@ def sanitize_timestamp(timestamp):
     return timeutils.normalize_time(timestamp)
 
 
-def stringify_timestamps(data):
-    """Stringify any datetime in given dict."""
-    isa_timestamp = lambda v: isinstance(v, datetime.datetime)
-    return dict((k, v.isoformat() if isa_timestamp(v) else v)
-                for (k, v) in six.iteritems(data))
-
-
 def dict_to_keyval(value, key_base=None):
     """Expand a given dict to its corresponding key-value pairs.
 
