@@ -62,6 +62,10 @@ function ceilometer_service_url {
 }
 
 
+function gnocchi_service_url {
+    echo "$GNOCCHI_SERVICE_PROTOCOL://$GNOCCHI_SERVICE_HOST:$GNOCCHI_SERVICE_PORT"
+}
+
 # _ceilometer_install_mongdb - Install mongodb and python lib.
 function _ceilometer_install_mongodb {
     # Server package is the same on all
@@ -127,7 +131,7 @@ function _ceilometer_config_apache_wsgi {
 function _ceilometer_prepare_coordination {
     if echo $CEILOMETER_COORDINATION_URL | grep -q '^memcached:'; then
         install_package memcached
-    elif [[ "${CEILOMETER_COORDINATOR_URL%%:*}" == "redis" || "${CEILOMETER_CACHE_BACKEND##*.}" == "redis" ]]; then
+    elif [[ "${CEILOMETER_COORDINATOR_URL%%:*}" == "redis" || "${CEILOMETER_CACHE_BACKEND##*.}" == "redis" || "${CEILOMETER_BACKEND}" == "gnocchi" ]]; then
         _ceilometer_install_redis
     fi
 }
@@ -179,9 +183,82 @@ function ceilometer_create_accounts {
         get_or_add_user_project_role "ResellerAdmin" "ceilometer" $SERVICE_PROJECT_NAME
     fi
 
+    if [ "$CEILOMETER_BACKEND" == "gnocchi" ]; then
+        create_service_user "gnocchi"
+        local gnocchi_service=$(get_or_create_service "gnocchi" \
+            "metric" "OpenStack Metric Service")
+        get_or_create_endpoint $gnocchi_service \
+            "$REGION_NAME" \
+            "$(gnocchi_service_url)" \
+            "$(gnocchi_service_url)" \
+            "$(gnocchi_service_url)"
+    fi
     export OS_CLOUD=$OLD_OS_CLOUD
 }
 
+
+function install_gnocchi {
+    echo_summary "Installing Gnocchi"
+    if python3_enabled; then
+        PY_VERS=${PYTHON3_VERSION}
+    else
+        PY_VERS=${PYTHON2_VERSION}
+    fi
+    # workaround for upper-constraints. pandas is uncapped but and numpy is.
+    sudo -H python${PY_VERS} -m pip install -U "gnocchi[file,${DATABASE_TYPE},keystone]>=3.1,<4.0"
+    recreate_database gnocchi
+    sudo install -d -o $STACK_USER -m 755 $GNOCCHI_CONF_DIR
+
+    [ ! -d $GNOCCHI_DATA_DIR ] && sudo mkdir -m 755 -p $GNOCCHI_DATA_DIR
+    sudo chown $STACK_USER $GNOCCHI_DATA_DIR
+
+    if [ -n "$GNOCCHI_COORDINATOR_URL" ]; then
+        iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
+    fi
+    iniset $GNOCCHI_CONF DEFAULT debug "$ENABLE_DEBUG_LOG_LEVEL"
+    iniset $GNOCCHI_CONF indexer url `database_connection_url gnocchi`
+    iniset $GNOCCHI_CONF storage driver file
+    iniset $GNOCCHI_CONF storage file_basepath $GNOCCHI_DATA_DIR/
+    iniset $GNOCCHI_CONF metricd metric_processing_delay "$GNOCCHI_METRICD_PROCESSING_DELAY"
+
+    iniset $GNOCCHI_CONF api auth_mode keystone
+    configure_auth_token_middleware $GNOCCHI_CONF gnocchi $GNOCCHI_AUTH_CACHE_DIR
+
+    sudo mkdir -p $GNOCCHI_AUTH_CACHE_DIR
+    sudo chown $STACK_USER $GNOCCHI_AUTH_CACHE_DIR
+    rm -f $GNOCCHI_AUTH_CACHE_DIR/*
+
+    gnocchi-upgrade --create-legacy-resource-types
+
+    # cleanup
+    sudo rm -f $GNOCCHI_WSGI_DIR/*.wsgi
+    sudo rm -f $(apache_site_config_for gnocchi)
+
+    sudo mkdir -p $GNOCCHI_WSGI_DIR
+
+    local gnocchi_apache_conf=$(apache_site_config_for gnocchi)
+    local venv_path=""
+    local script_name=$GNOCCHI_SERVICE_PREFIX
+
+    echo "from gnocchi.rest import app" | sudo tee -a $GNOCCHI_WSGI_DIR/app
+    echo "application = app.build_wsgi_app()" | sudo tee -a $GNOCCHI_WSGI_DIR/app
+
+    sudo cp $CEILOMETER_DIR/devstack/apache-ceilometer.template $gnocchi_apache_conf
+    sudo sed -e "
+        s|ceilometer-api|gnocchi|g;
+        s|ceilometer|gnocchi|g;
+        s|%PORT%|$GNOCCHI_SERVICE_PORT|g;
+        s|%APACHE_NAME%|$APACHE_NAME|g;
+        s|%WSGIAPP%|$GNOCCHI_WSGI_DIR/app|g;
+        s|%USER%|$STACK_USER|g;
+        s|%VIRTUALENV%|$venv_path|g
+    " -i $gnocchi_apache_conf
+
+    if [ -n "$GNOCCHI_COORDINATOR_URL" ]; then
+        iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
+    fi
+}
+
 # Activities to do before ceilometer has been installed.
 function preinstall_ceilometer {
     echo_summary "Preinstall not in virtualenv context. Skipping."
@@ -364,11 +441,6 @@ function init_ceilometer {
                 $CEILOMETER_BIN_DIR/ceilometer-upgrade --skip-gnocchi-resource-types
             fi
         fi
-        if is_service_enabled gnocchi ; then
-            if [ "$CEILOMETER_BACKEND" = 'gnocchi' ]; then
-                $CEILOMETER_BIN_DIR/ceilometer-upgrade --skip-metering-database
-            fi
-        fi
     fi
 }
 
@@ -378,10 +450,12 @@ function init_ceilometer {
 # installed. The context is not active during preinstall (when it would
 # otherwise makes sense to do the backend services).
 function install_ceilometer {
-    if is_service_enabled ceilometer-acentral ceilometer-acompute ceilometer-anotification ; then
+    if is_service_enabled ceilometer-acentral ceilometer-acompute ceilometer-anotification gnocchi-api gnocchi-metricd; then
         _ceilometer_prepare_coordination
     fi
 
+    [ "$CEILOMETER_BACKEND" = 'gnocchi' ] && install_gnocchi
+
     if is_service_enabled ceilometer-collector ceilometer-api; then
         _ceilometer_prepare_storage_backend
     fi
@@ -391,7 +465,14 @@ function install_ceilometer {
     fi
 
     install_ceilometerclient
-    setup_develop $CEILOMETER_DIR
+
+    case $CEILOMETER_BACKEND in
+        mongodb) extra=mongo;;
+        gnocchi) extra=gnocchi;;
+        mysql) extra=mysql;;
+        postgresql) extra=postgresql;;
+    esac
+    setup_develop $CEILOMETER_DIR $extra
     sudo install -d -o $STACK_USER -m 755 $CEILOMETER_CONF_DIR
 }
 
@@ -408,6 +489,21 @@ function install_ceilometerclient {
 
 # start_ceilometer() - Start running processes, including screen
 function start_ceilometer {
+
+    if [ "$CEILOMETER_BACKEND" = "gnocchi" ] ; then
+        enable_apache_site gnocchi
+        restart_apache_server
+        tail_log gnocchi /var/log/$APACHE_NAME/gnocchi.log
+        tail_log gnocchi-api /var/log/$APACHE_NAME/gnocchi-access.log
+        echo "Waiting for gnocchi-api to start..."
+        if ! timeout $SERVICE_TIMEOUT sh -c "while ! curl -v --max-time 5 --noproxy '*' -s $(gnocchi_service_url)/v1/resource/generic ; do sleep 1; done"; then
+            die $LINENO "gnocchi-api did not start"
+        fi
+
+        run_process gnocchi-metricd "/usr/local/bin/gnocchi-metricd --config-file $GNOCCHI_CONF"
+        $CEILOMETER_BIN_DIR/ceilometer-upgrade --skip-metering-database
+    fi
+
     run_process ceilometer-acentral "$CEILOMETER_BIN_DIR/ceilometer-polling --polling-namespaces central --config-file $CEILOMETER_CONF"
     run_process ceilometer-anotification "$CEILOMETER_BIN_DIR/ceilometer-agent-notification --config-file $CEILOMETER_CONF"
     run_process ceilometer-aipmi "$CEILOMETER_BIN_DIR/ceilometer-polling --polling-namespaces ipmi --config-file $CEILOMETER_CONF"
@@ -473,6 +569,8 @@ if is_service_enabled ceilometer; then
         init_ceilometer
         # Start the services
         start_ceilometer
+    elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
+        iniset $TEMPEST_CONFIG telemetry alarm_granularity $CEILOMETER_ALARM_GRANULARITY
     fi
 
     if [[ "$1" == "unstack" ]]; then
